"""
Dilbert Scraper
~~~~~~~~~~~~~~~~~~~

A basic scraper for Dilbert.

:copyright: (c) 2022 Omkaar
:license: MIT, see LICENSE for more details.
"""


__title__ = "dilbert"
__author__ = "Omkaar"
__license__ = "MIT"
__copyright__ = "Copyright 2022 Omkaar"
__version__ = "1.1.5"


from .comic import *
from .utils import *
